//*********************************************************************************
//                               asus.cc
//---------------------------------------------------------------------------------
// Hugo Mercier <hugo.mercier@laposte.net> (c) 2005 - Initial Author (Auteur initial)
//
// GNU GENERAL PUBLIC LICENSE - See COPYING. (COPYING-fr, version française)
//*********************************************************************************

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/io.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>


struct AsusFlashHeader
{
  char sign[16];
  unsigned char version[2];
  unsigned long enableP;
  unsigned long disableP;
};

AsusFlashHeader head;

int main(int argc, char*argv[])
{

  int r = iopl(3);
  if (r == -1) {
    fprintf(stderr, "Can't access I/O\n");
    return 1;
  }

  // map the F000 segment => address 000F0000
  int fd = open("/dev/mem", O_RDONLY);

  if (fd == -1) {
    fprintf(stderr,"Can't access /dev/mem\n");
    return 1;
  }  

  int offset;
  int found;
  char *reg1;
  found=0;
  reg1 = (char*)mmap(0, 65536, PROT_EXEC | PROT_READ, MAP_PRIVATE, fd, (off_t) 0xF0000);
  if (reg1 == (void*)-1) {
    fprintf(stderr,"Can't mmap() (%d)\n", errno);
    return 1;
  }
  
  // look for signature
  char sigToFind[]="ASUS_FLASH";
  for (offset=0; offset < 65520; offset++) {
    char* sig=reg1 + offset;
    if (!strncmp(sig, sigToFind, strlen(sigToFind))) {
      found = 1;
      break;
    }
  }
  
  if (!found) {
    fprintf(stderr, "ASUS Flash signature not found\n");
    return 1;
  }
  
  memcpy(&head.version, reg1 + offset+16,2);
  memcpy(&head.disableP, reg1 + offset+18,4);
  memcpy(&head.enableP, reg1 + offset+24,4);
 
  printf("ASUS Flash version %d.%d disable : 0x%08X enable : 0x%08X\n",
	 head.version[0],
	 head.version[1],
	 head.disableP,
	 head.enableP);
 
  close(fd);

  return 0;
}

